#Import Modules
import os, pygame, random
from pygame.locals import *

from helper import *
from level import Level

class Monster(pygame.sprite.Sprite):
    def __init__(self):
        random.seed()
        pygame.sprite.Sprite.__init__(self)
        self.image,self.rect = load_image('lizardmanB1.gif',-1) #this is to get the 32x32 rect
        self.aiCount = 0
        self.posx = 640.0 #position of monster
        self.posy = 670.0 #position of monster
        self.rect.topleft = self.posx, self.posy
        self.speed = 2.8 # lizard's speed?
        
        self.worldmaxy = 0
        self.worldmaxx = 0

        
        self.frameCount = 0
        self.state = 0 # 0 = stand, 1 = move
        self.direction = 0 # 0 = bottom, 1 = bottomright, 2 = right, 3 = topright
                           # 4 = top, 5 = topleft, 6 = left, 7 = bottomleft

        #lets setup initial images for player
        self.runB = []
        runB1, null = load_image('lizardmanB1.gif',-1)
        runB2, null = load_image('lizardmanB2.gif',-1)
        self.runB.append(runB1)
        self.runB.append(runB2)

        self.runBR = []
        runBR1, null = load_image('lizardmanBR1.gif',-1)
        runBR2, null = load_image('lizardmanBR2.gif',-1)
        self.runBR.append(runBR1)
        self.runBR.append(runBR2)

        self.runR = []
        runR1, null = load_image('lizardmanR1.gif',-1)
        runR2, null = load_image('lizardmanR2.gif',-1)
        self.runR.append(runR1)
        self.runR.append(runR2)

        self.runTR = []
        runTR1, null = load_image('lizardmanTR1.gif',-1)
        runTR2, null = load_image('lizardmanTR2.gif',-1)
        self.runTR.append(runTR1)
        self.runTR.append(runTR2)

        self.runT = []
        runT1, null = load_image('lizardmanT1.gif',-1)
        runT2, null = load_image('lizardmanT2.gif',-1)
        self.runT.append(runT1)
        self.runT.append(runT2)

        self.runTL = []
        runTL1 = pygame.transform.flip(runTR1,1,0)
        runTL2 = pygame.transform.flip(runTR2,1,0)
        self.runTL.append(runTL1)
        self.runTL.append(runTL2)

        self.runL = []
        runL1 = pygame.transform.flip(runR1,1,0)
        runL2 = pygame.transform.flip(runR2,1,0)
        self.runL.append(runL1)
        self.runL.append(runL2)

        self.runBL = []
        runBL1 = pygame.transform.flip(runBR1,1,0)
        runBL2 = pygame.transform.flip(runBR2,1,0)
        self.runBL.append(runBL1)
        self.runBL.append(runBL2)
        # all 8 positions have been updated, woop

    def update(self, curLevel, worldx, worldy):
        self.updateAI()
        if self.state > 0:
            if self.frameCount == 0:        # frame 0
                if self.direction == 0:
                    self.image = self.runB[0]
                elif self.direction == 1:
                    self.image = self.runBR[0]
                elif self.direction == 2:
                    self.image = self.runR[0]
                elif self.direction == 3:
                    self.image = self.runTR[0]
                elif self.direction == 4:
                    self.image = self.runT[0]
                elif self.direction == 5:
                    self.image = self.runTL[0]
                elif self.direction == 6:
                    self.image = self.runL[0]
                elif self.direction == 7:
                    self.image = self.runBL[0]
            elif self.frameCount == 10:     # frame 1
                if self.direction == 0:
                    self.image = self.runB[1]
                elif self.direction == 1:
                    self.image = self.runBR[1]
                elif self.direction == 2:
                    self.image = self.runR[1]
                elif self.direction == 3:
                    self.image = self.runTR[1]
                elif self.direction == 4:
                    self.image = self.runT[1]
                elif self.direction == 5:
                    self.image = self.runTL[1]
                elif self.direction == 6:
                    self.image = self.runL[1]
                elif self.direction == 7:
                    self.image = self.runBL[1]
            self.frameCount = (self.frameCount + 1) % 20

            self.monsterCollide(curLevel)
        self.rect.topleft = (int(self.posx)-worldx, int(self.posy)-worldy)

    def monsterCollide(self, level):
        #debug
        totalx = self.posx
        totaly = self.posy
        extra = int(self.speed)
        if self.direction == 0:
            # check for wall!
            bottom = []
            for i in range(0+extra,32-extra):
                bottom.append(level.tileAt(totalx+i,totaly+32+extra))
                                # one more than character's 32x32
            if 'W' in bottom:      # for now, we had a wall, do not move
                return
            if self.posy < self.worldmaxy:
                self.posy += self.speed
        elif self.direction == 1:
            # check for wall!
            bottom = []
            for i in range(0+extra,32-extra):
                bottom.append(level.tileAt(totalx+i,totaly+32+extra))
                                # one more than character's 32x32
            if 'W' not in bottom:      # for now, we had a wall, do not move
                if self.posy < self.worldmaxy:        
                    self.posy += self.speed * 0.7
            # check for wall!
            right = []
            for i in range(0+extra,16-extra):
                right.append(level.tileAt(totalx+32+extra,totaly+16+i))
                                # one more than character's 32x32
            if 'W' in right:      # for now, we had a wall, do not move
                return
            
            if self.posx < self.worldmaxx:
                self.posx += self.speed * 0.7
        elif self.direction == 2:
            # check for wall!
            right = []
            for i in range(0+extra,16-extra):
                right.append(level.tileAt(totalx+32+extra,totaly+16+i))
                                # one more than character's 32x32
            if 'W' in right:      # for now, we had a wall, do not move
                return
            
            if self.posx < self.worldmaxx:
                self.posx += self.speed
        elif self.direction == 3:
            # check for wall above character
            top = []
            for i in range(0+extra,32-extra):
                top.append(level.tileAt(totalx+i,totaly+16-extra))
                                # one more than character's 32x32
            if 'W' not in top:      # for now, we had a wall, do not move
                if self.posy > 0:
                    self.posy -= self.speed * 0.7
            # check for wall!
            right = []
            for i in range(0+extra,16-extra):
                right.append(level.tileAt(totalx+32+extra,totaly+16+i))
                                # one more than character's 32x32
            if 'W' in right:      # for now, we had a wall, do not move
                return
            if self.posx < self.worldmaxx:
                self.posx += self.speed * 0.7
        elif self.direction == 4:
            # check for wall above character
            top = []
            for i in range(0+extra,32-extra):
                top.append(level.tileAt(totalx+i,totaly+16-extra))
                                # one more than character's 32x32
            if 'W' in top:      # for now, we had a wall, do not move
                return
            if self.posy > 0:
                self.posy -= self.speed
        elif self.direction == 5:
            # check for wall above character
            top = []
            for i in range(0+extra,32-extra):
                top.append(level.tileAt(totalx+i,totaly+16-extra))
                                # one more than character's 32x32
            if 'W' not in top:      # for now, we had a wall, do not move
                if self.posy > 0:
                    self.posy -= self.speed * 0.7
            # check for wall!
            left = []
            for i in range(0+extra,16-extra):
                left.append(level.tileAt(totalx-extra,totaly+16+i))
                                # one more than character's 32x32
            if 'W' in left:      # for now, we had a wall, do not move
                return
            if self.posx > 0:
                self.posx -= self.speed * 0.7
        elif self.direction == 6:
            left = []
            for i in range(0+extra,16-extra):
                left.append(level.tileAt(totalx-extra,totaly+16+i))
                                # one more than character's 32x32
            if 'W' in left:      # for now, we had a wall, do not move
                return
            if self.posx > 0:
                self.posx -= self.speed
        elif self.direction == 7:
            # check for wall!
            bottom = []
            for i in range(0+extra,32-extra):
                bottom.append(level.tileAt(totalx+i,totaly+32+extra))
                                # one more than character's 32x32
            if 'W' not in bottom:      # for now, we had a wall, do not move
                if self.posy < self.worldmaxy:
                    self.posy += self.speed * 0.7
            left = []
            for i in range(0+extra,16-extra):
                left.append(level.tileAt(totalx-extra,totaly+16+i))
                                # one more than character's 32x32
            if 'W' in left:      # for now, we had a wall, do not move
                return
            if self.posx > 0:
                self.posx -= self.speed * 0.7

    #define our max X & Y for a world set
    def setMaxX(self, maxX):
        self.worldmaxx = maxX

    def setMaxY(self, maxY):
        self.worldmaxy = maxY

    def updateAI(self):
        if self.aiCount < 10:
            self.aiCount = self.aiCount + 1
        else:
            self.aiCount = 0
            self.state = random.randint(0,2)
            self.direction = random.randint(0,8)
